@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2013 - 2014 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2013 - 2018 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #ifndef _LIBMBIM_GLIB_MBIM_CID_H_
@@ -219,6 +219,80 @@ typedef enum {
     MBIM_CID_PROXY_CONTROL_CONFIGURATION = 1
 } MbimCidProxyControl;
 
+/**
+ * MbimCidQmi:
+ * @MBIM_CID_QMI_UNKNOWN: Unknown command.
+ * @MBIM_CID_QMI_MSG: Send QMI request and receive QMI response.
+ *
+ * MBIM commands in the %MBIM_SERVICE_QMI service.
+ */
+typedef enum {
+    MBIM_CID_QMI_UNKNOWN = 0,
+    MBIM_CID_QMI_MSG     = 1
+} MbimCidQmi;
+
+/**
+ * MbimCidAtds:
+ * @MBIM_CID_ATDS_UNKNOWN: Unknown command.
+ * @MBIM_CID_ATDS_SIGNAL: Radio signal information.
+ * @MBIM_CID_ATDS_LOCATION: Cell location information.
+ * @MBIM_CID_ATDS_OPERATORS: Operator selection.
+ * @MBIM_CID_ATDS_RAT: Radio Access Technology selection.
+ * @MBIM_CID_ATDS_REGISTER_STATE: Registration state.
+ *
+ * MBIM commands in the %MBIM_SERVICE_ATDS service.
+ */
+typedef enum {
+    MBIM_CID_ATDS_UNKNOWN        = 0,
+    MBIM_CID_ATDS_SIGNAL         = 1,
+    MBIM_CID_ATDS_LOCATION       = 2,
+    MBIM_CID_ATDS_OPERATORS      = 3,
+    MBIM_CID_ATDS_RAT            = 4,
+    MBIM_CID_ATDS_REGISTER_STATE = 9,
+} MbimCidAtds;
+
+/**
+ * MbimCidIntelFirmwareUpdate:
+ * @MBIM_CID_INTEL_FIRMWARE_UPDATE_UNKNOWN: Unknown command.
+ * @MBIM_CID_INTEL_FIRMWARE_UPDATE_MODEM_REBOOT: Reboot modem for firmware update.
+ *
+ * MBIM commands in the %MBIM_SERVICE_INTEL_FIRMWARE_UPDATE service.
+ */
+typedef enum {
+    MBIM_CID_INTEL_FIRMWARE_UPDATE_UNKNOWN      = 0,
+    MBIM_CID_INTEL_FIRMWARE_UPDATE_MODEM_REBOOT = 1,
+} MbimCidIntelFirmwareUpdate;
+
+/**
+ * MbimCidMsBasicConnectExtensions:
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_UNKNOWN: Unknown command.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_PROVISIONED_CONTEXTS: Provisioned contexts (v2).
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_NETWORK_BLACKLIST: Network blacklist.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_LTE_ATTACH_CONFIGURATION: LTE attach configuration.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_LTE_ATTACH_STATUS: LTE attach status.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_SYS_CAPS: System capabilities.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_CAPS: Device capabilities (v2).
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_SLOT_MAPPINGS: Device slot mappings.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_SLOT_INFO_STATUS: Slot info status.
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_PCO: Protocol configuration operations (PCO).
+ * @MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_RESET: Device reset.
+ *
+ * MBIM commands in the %MBIM_SERVICE_MS_BASIC_CONNECT_EXTENSIONS service.
+ */
+typedef enum {
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_UNKNOWN                  = 0,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_PROVISIONED_CONTEXTS     = 1,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_NETWORK_BLACKLIST        = 2,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_LTE_ATTACH_CONFIGURATION = 3,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_LTE_ATTACH_STATUS        = 4,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_SYS_CAPS                 = 5,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_CAPS              = 6,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_SLOT_MAPPINGS     = 7,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_SLOT_INFO_STATUS         = 8,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_PCO                      = 9,
+    MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_DEVICE_RESET             = 10,
+} MbimCidMsBasicConnectExtensions;
+
 /* Command helpers */
 
 gboolean     mbim_cid_can_set       (MbimService service,
