@@ -172,7 +172,7 @@ print_version_and_exit (void)
 {
     g_print ("\n"
              PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (C) 2013-2018 Aleksander Morgado\n"
+             "Copyright (C) 2013-2019 Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -278,6 +278,9 @@ device_open_ready (MbimDevice   *dev,
     case MBIM_SERVICE_INTEL_FIRMWARE_UPDATE:
         mbimcli_intel_firmware_update_run (dev, cancellable);
         return;
+    case MBIM_SERVICE_MS_BASIC_CONNECT_EXTENSIONS:
+        mbimcli_ms_basic_connect_extensions_run (dev, cancellable);
+        return;
     default:
         g_assert_not_reached ();
     }
@@ -355,6 +358,9 @@ parse_actions (void)
     } else if (mbimcli_intel_firmware_update_options_enabled ()) {
         service = MBIM_SERVICE_INTEL_FIRMWARE_UPDATE;
         actions_enabled++;
+    } else if (mbimcli_ms_basic_connect_extensions_options_enabled ()) {
+        service = MBIM_SERVICE_MS_BASIC_CONNECT_EXTENSIONS;
+        actions_enabled++;
     }
 
     /* Noop */
@@ -400,6 +406,8 @@ int main (int argc, char **argv)
                                 mbimcli_atds_get_option_group ());
     g_option_context_add_group (context,
                                 mbimcli_intel_firmware_update_get_option_group ());
+    g_option_context_add_group (context,
+                                mbimcli_ms_basic_connect_extensions_get_option_group ());
     g_option_context_add_main_entries (context, main_entries, NULL);
     if (!g_option_context_parse (context, &argc, &argv, &error)) {
         g_printerr ("error: %s\n",
